from nicegui import ui
from ..type_view import TypeView


class AssetView(TypeView):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @property
    def asset(self):
        return self.component

    @ui.refreshable
    def ui_content(self):
        with ui.card().classes('w-full h-full').props('color="blue-800" keep-color') as self.card:
            with ui.item().classes('w-full h-full'):
                with ui.item_section():
                    self.checkbox = ui.checkbox()
                with ui.item_section():
                    ui.label(self.asset.name)
                with ui.item_section():
                    ui.label(str(self.asset.file_size / 1024))
                with ui.item_section():
                    ui.label(str(self.asset.last_modified))
                with ui.item_section():
                    dl_button = ui.button(icon='download', on_click=self.download).classes('q-ml-auto')

    def download(self, event):
        ui.download(f'assets/{self.asset.name}')



from typing import Union
from .user import UserManager

from PySimultan2.object_mapper import PythonMapper
from PySimultan2.data_model import DataModel
from .method_mapper import method_mapper, mapped_method
from .logging import logger

from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from ..app.home import ProjectManager
    from ..views.view_manager import ViewManager
    from ..views.geometry_view import GeometryManager
    from .navigation import Navigation
    from ..views.asset_view import AssetManager


mapper = PythonMapper()
method_mapper.mapper = mapper
mapper.method_mapper = method_mapper

# data_model: Union[DataModel, None] = None
# navigation: Union['Navigation', None] = None
# view_manager: Union['ViewManager', None] = None
# project_manager: Union['ProjectManager', None] = None
# asset_manager: Union['AssetManager', None] = None
# geometry_manager: Union['GeometryManager', None] = None
#
# navigation_drawer = None
# project_tab = None
# detail_view = None
# home_tab = None



import os
from typing import Union, Optional

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .method_mapper import MethodMapper
    from PySimultan2.data_model import DataModel
    from PySimultan2.object_mapper import PythonMapper

initial_user_name = os.environ.get('INITIAL_USER_NAME', 'admin')
initial_user_email = os.environ.get('INITIAL_USER_EMAIL', 'example@test.de')
initial_user_password = os.environ.get('INITIAL_USER_PASSWORD', 'admin')


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class DetailHistory:

    max_history_entries = 10

    def __init__(self):
        self.detail_history = []
        self.detail_history_index = 0
        self.current_detail = None

    def add_item(self, item):

        if len(self.detail_history) > 0:
            if item is self.detail_history[self.detail_history_index]:
                return

        if len(self.detail_history) > 0 and self.detail_history_index < (len(self.detail_history) - 1):
            self.detail_history = self.detail_history[:self.detail_history_index]
            self.detail_history_index = len(self.detail_history) - 1

        self.detail_history.append(item)
        if len(self.detail_history) > self.max_history_entries:
            self.detail_history.pop(0)
        self.detail_history_index = len(self.detail_history) - 1
        self.current_detail = item

    def get_current_detail(self):
        return self.current_detail

    def get_previous_detail(self):
        if self.detail_history_index > 0:
            return self.detail_history[self.detail_history_index - 1]

    def get_next_detail(self):
        if self.detail_history_index < (len(self.detail_history) - 1):
            return self.detail_history[self.detail_history_index + 1]

    def __len__(self):
        return len(self.detail_history)

    def move_next(self):
        if self.detail_history_index < len(self.detail_history) - 1:
            self.detail_history_index += 1
            return self.detail_history[self.detail_history_index]

    def move_previous(self):
        if self.detail_history_index > 0:
            self.detail_history_index -= 1
            return self.detail_history[self.detail_history_index]


class User:

    def __init__(self, *args, **kwargs):

        from ..core import method_mapper, mapper

        self._project_manager = None
        self._grid_view = None
        self._view_manager = None
        self._asset_manager = None
        self._geometry_manager = None
        self._array_manager = None
        self._navigation = None
        self._data_model = None
        self._ui_log = None
        self._log_tab = None

        self.user_manager = kwargs.get('user_manager', None)
        self.detail_history = DetailHistory()

        self.name = kwargs.get('name', None)
        self.email = kwargs.get('email', None)
        self.password = kwargs.get('password', None)

        self.mapper: Optional[PythonMapper] = kwargs.get('mapper', mapper)
        self.method_mapper: Optional[MethodMapper] = kwargs.get('method_mapper', method_mapper)
        self.data_model: Union[DataModel, None] = kwargs.get('data_model', None)

        self.navigation_drawer = kwargs.get('navigation_drawer', None)
        self.project_tab = kwargs.get('project_tab', None)
        self.log_tab = kwargs.get('log_tab', None)
        self.detail_view = kwargs.get('detail_view', None)
        self.home_tab = kwargs.get('home_tab', None)

    @property
    def data_model(self):
        return self._data_model

    @data_model.setter
    def data_model(self, value):
        self._data_model = value
        if self.detail_history is not None:
            self.detail_history = DetailHistory()

    @property
    def grid_view(self):
        if self._grid_view is None:
            from ..views.grid_view import GridView
            self._grid_view = GridView()
        return self._grid_view

    @property
    def project_manager(self):
        if self._project_manager is None:
            from .project_manager import ProjectManager
            self._project_manager = ProjectManager(user_manager=self.user_manager,
                                                   mapper=self.mapper)
        return self._project_manager

    @property
    def view_manager(self):
        # if self._view_manager is None:
        #     from ..views.view_manager import ViewManager
        #     self._view_manager = ViewManager(mapper=self.mapper,
        #                                      parent=self.project_manager)
        return self._view_manager

    @property
    def asset_manager(self):
        if self._asset_manager is None:
            from ..views.asset_view import AssetManager
            self._asset_manager = AssetManager()
        return self._asset_manager

    @property
    def array_manager(self):
        if self._array_manager is None:
            from ..views.array_view import ArrayManager
            self._array_manager = ArrayManager()
        return self._array_manager

    @property
    def geometry_manager(self):
        if self._geometry_manager is None:
            from ..views.geometry_view import GeometryManager
            self._geometry_manager = GeometryManager(data_model=self.data_model)
        return self._geometry_manager

    @property
    def navigation(self):
        # if self._navigation is None:
        #     from .navigation import Navigation
        #     self._navigation = Navigation()
        return self._navigation

    @property
    def log_tab(self):
        return self._log_tab

    @log_tab.setter
    def log_tab(self, value):
        self._log_tab = value

        if self._log_tab is not None and self._ui_log is None:
            self.add_ui_log()

    @property
    def ui_log(self):
        if self._ui_log is None and self.log_tab is not None:
            self.add_ui_log()
        return self._ui_log

    @property
    def logger(self):
        return self.ui_log.logger

    def logout(self):
        self.data_model = None

    def add_ui_log(self):
        if self.log_tab is None:
            return
        from .ui_log import UILog
        with self.log_tab:
            self._ui_log = UILog()
            self._ui_log.ui_content()


class Admin(User):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class UserManager(metaclass=Singleton):

    def __init__(self):
        self.users = {}
        self.create_initial_users()

    def create_initial_users(self):
        admin = Admin(name=initial_user_name,
                      email=initial_user_email,
                      password=initial_user_password,
                      user_manager=self)

        self.users[initial_user_name] = admin

        user_1 = User(name=initial_user_name + '_2',
                      email=initial_user_email + '_2',
                      password=initial_user_password + '_2',
                      user_manager=self)

        self.users[initial_user_name + '_2'] = user_1

    def authenticate(self, username, password):
        if username in self.users and self.users[username].password == password:
            return True
        return False

    def __getitem__(self, key):
        return self.users[key]



import numpy as np
import pandas as pd
from pandas import DataFrame
# import pandas as pd

from nicegui import ui, app
from logging import getLogger
from PySimultan2.simultan_object import SimultanObject
from PySimultan2.geometry import GeometryModel
from PySimultan2.default_types import ComponentDictionary, ComponentList

from SIMULTAN.Data.MultiValues import (SimMultiValueField3D, SimMultiValueField3DParameterSource, SimMultiValueBigTable,
                                       SimMultiValueBigTableHeader, SimMultiValueBigTableParameterSource)

from .. import user_manager

from .pandas_df_view import DataFrameDetailView
from .numpy_view import NDArrayDetailView
from .geometry_view.geometry_view import GeometryDetailView
from .component_list_view.component_list_view import ListDetailView
from .component_dict_view.component_dict_view import DictDetailView
from .mapped_cls.mapped_cls_view import MappedClsDetailView


logger = getLogger('py_simultan_ui')


def show_next_detail(*args, **kwargs):
    user = user_manager[app.storage.user['username']]
    show_detail(value=user.detail_history.move_next(), *args, **kwargs)


def show_previous_detail(*args, **kwargs):
    user = user_manager[app.storage.user['username']]
    show_detail(value=user.detail_history.move_previous(), *args, **kwargs)


def show_detail(value, *args, **kwargs):
    user = user_manager[app.storage.user['username']]
    user.detail_history.add_item(value)
    # current_detail = user.detail_history[-1] if user.detail_history else None
    logger.debug(f'Showing details for {value}')
    user.detail_view.clear()
    with user.detail_view:
        with ui.card().classes('w-full'):
            with ui.row():
                previous_detail = user.detail_history.get_previous_detail()
                if previous_detail is not None:
                    ui.button(on_click=show_previous_detail,
                              icon='arrow_back').classes('q-mr-md')
                ui.space()
                next_detail = user.detail_history.get_next_detail()
                if next_detail is not None:
                    ui.button(on_click=show_next_detail,
                              icon='arrow_forward').classes('q-mr-md')

            user.current_detail = value

            if isinstance(value, SimMultiValueBigTable):
                detail_view = DataFrameDetailView(component=value, **kwargs)
            elif isinstance(value, SimMultiValueField3D):
                detail_view = NDArrayDetailView(component=value, **kwargs)
            elif isinstance(value, GeometryModel):
                detail_view = GeometryDetailView(component=value, **kwargs)
            elif isinstance(value, ComponentList):
                detail_view = ListDetailView(component=value, **kwargs)
            elif isinstance(value, ComponentDictionary):
                detail_view = DictDetailView(component=value, **kwargs)
            elif isinstance(value, SimultanObject):
                detail_view = MappedClsDetailView(component=value, **kwargs)
            else:
                cls_view = user.view_manager.cls_views.get(value.__class__, None)
                if cls_view is None:
                    ui.label(f'No view for {value.__class__}')
                    return
                type_view = cls_view.get('type_view', None)
                if type_view is None:
                    ui.label(f'No type view for {value.__class__}')
                    return
                if not hasattr(type_view, 'detail_view'):
                    ui.label(f'No detail view for {value.__class__}')
                    return
                detail_view = type_view.detail_view(component=value, **kwargs)

            detail_view.ui_content()



from contextlib import contextmanager
from .menue import menu
from nicegui import app, ui, context
from .. import user_manager

from . import home
from . import project

from .. import core


@contextmanager
def frame(navtitle: str):

    """Custom page frame to share the same styling and behavior across all pages"""
    # ui.colors(primary='#da291c', secondary='#941c13', accent='#941c13', positive='#53B689')
    with ui.header().classes('justify-between'):

        user = user_manager.users[app.storage.user['username']]

        # ui.image('web_ui/src/static_files/A1_Digital_identifier_pos_RGB.png').classes('w-32')

        with ui.column():
            ui.label('PY Simultan').classes('font-bold text-white text-xl')
            ui.button(on_click=lambda: (user_manager.users[app.storage.user['username']].logout(),
                                        app.storage.user.clear(),
                                        ui.navigate.to('/login')), icon='logout')

        with ui.column():
            ui.label(f'Hello {user.name}').classes('text-white text-xl')

        # with ui.expansion('Menu', icon='select'):
        with ui.column():
            with ui.tabs() as tabs:
                # ui.button('Mapped Classes', icon='checklist_rtl', on_click=lambda: left_drawer.toggle())
                ui.tab('Home', icon='home')
                ui.tab('Project', icon='edit')
                ui.tab('Logs', icon='logs')

                # ui.button(on_click=lambda: (app.storage.user.clear(),
                #                             ui.open('/login')),
                #           icon='logout').props('outline round').classes('text-black')
                #


                # with ui.button(icon='menu'):
                #     with ui.menu() as menu:
                #         ui.menu_item('Logout',
                #                      icon='logout',
                #                      on_click=lambda: (app.storage.user.clear(), ui.open('/login')))

                # ui.space()
                # if app.storage.user.get('username', False):
                #     ui.label(f'{app.storage.user["username"]}').classes('mr-auto text-2xl')

    # with ui.left_drawer(bordered=True,
    #                     top_corner=False,
    #                     elevated=True,
    #                     fixed=True,
    #                     value=False).classes('bg-blue-100') as left_drawer:
    #     user.navigation_drawer = left_drawer

    # with ui.splitter() as splitter:
    #     with splitter.after:
    #         with ui.right_drawer(bordered=True,
    #                              top_corner=False,
    #                              elevated=True,
    #                              fixed=True,
    #                              value=False).classes('w-full h-full') as detail_view:
    #             core.detail_view = detail_view

    with ui.tab_panels(tabs, value='Home').classes('w-full h-full'):
        with ui.tab_panel('Project').classes('w-full h-full') as project_full_tab:
            with ui.splitter(value=60, limits=(10, 90)).classes('w-full h-full') as splitter:
                with splitter.before as project_tab:
                    context.client.content.classes('h-[95vh]')
                    user.project_tab = project_tab
                with splitter.after:
                    user.detail_view = ui.row().classes('w-full h-full')

        user.home_tab = ui.tab_panel('Home').classes('w-full h-full')

        with ui.tab_panel('Logs').classes('w-full h-full'):
            with ui.card().classes('w-full h-full') as logs_tab:
                user.log_tab = logs_tab

    with user.home_tab:
        home.content()
